@@ -348,14 +348,16 @@ If an unsupported TYPE is requested, the function will return NIL.
                             (sb-c:fun-info-optimizer . sb-c:optimizer)
                             (sb-c:fun-info-ir2-convert . sb-c:ir2-convert)
                             (sb-c::fun-info-ir2-hook . sb-c::ir2-hook)
-                            (sb-c::fun-info-stack-allocate-result
-                             . sb-c::stack-allocate-result)
-                            (sb-c::fun-info-constraint-propagate
-                             . sb-c::constraint-propagate)
-                            (sb-c::fun-info-constraint-propagate-if
-                             . sb-c::constraint-propagate-if)
-                            (sb-c::fun-info-call-type-deriver
-                             . sb-c::call-type-deriver))))
+                            (sb-c::fun-info-stack-allocate-result . sb-c::stack-allocate-result)
+                            (sb-c::fun-info-constraint-propagate . sb-c::constraint-propagate)
+                            (sb-c::fun-info-constraint-propagate-if . sb-c::constraint-propagate-if)
+                            (sb-c::fun-info-constraint-propagate-back . sb-c::constraint-propagate-back)
+                            (sb-c::fun-info-constraint-propagate-result . sb-c::constraint-propagate-result)
+                            (sb-c::fun-info-equality-constraint . sb-c::equality-constraint)
+                            (sb-c::fun-info-folder . sb-c::folder)
+                            (sb-c::fun-info-externally-checkable-type . sb-c::externally-checkable-type)
+                            (sb-c::fun-info-constants . sb-c::constants)
+                            (sb-c::fun-info-call-type-deriver . sb-c::call-type-deriver))))
               (loop for (reader . name) in otypes
                     for fn = (funcall reader fun-info)
                     when fn collect
@@ -610,37 +612,81 @@ or a method combination name."
 ;;; XREF facility
 
 #-(and system-tlabs (not mark-region-gc))
-(defun collect-xref (wanted-kind wanted-name)
-  (let ((result '()))
-    (sb-c:map-simple-funs
-     (lambda (name fun)
-       (binding* ((xrefs (%simple-fun-xrefs fun) :exit-if-null))
-         (sb-c:map-packed-xref-data
-          (lambda (xref-kind xref-name xref-form-number)
-            (when (and (eq xref-kind wanted-kind)
-                       (equal xref-name wanted-name))
-              (let ((source-location (find-function-definition-source fun)))
-                ;; Use the more accurate source path from the xref
-                ;; entry.
-                (setf (definition-source-form-number source-location)
-                      xref-form-number)
-                (let ((name (cond ((sb-c::transform-p name)
-                                   (let ((fun-name (%fun-name fun)))
-                                     (append (if (consp fun-name)
-                                                 fun-name
-                                                 (list fun-name))
-                                             (let* ((type (sb-c::transform-type name))
-                                                    (type-spec (type-specifier type)))
-                                               (and (sb-kernel:fun-type-p type)
-                                                    (list (second type-spec)))))))
-                                  ((sb-c::vop-info-p name)
-                                   (list 'sb-c:define-vop
-                                         (sb-c::vop-info-name name)))
-                                  (t
-                                   name))))
-                  (push (cons name source-location) result)))))
-          xrefs))))
-    result))
+(progn
+  (labels ((functoid-simple-fun (functoid)
+             ;; looks like this is supposed to ignore INTERPRETED-FUNCTION ?
+             (typecase functoid
+               (simple-fun functoid)
+               (closure
+                (let ((fun (%closure-fun functoid)))
+                  (if (and (eq (%fun-name fun) 'sb-impl::encapsulation))
+                      (functoid-simple-fun
+                       (sb-impl::encapsulation-info-definition
+                        (sb-impl::encapsulation-info functoid)))
+                      fun))))))
+    (defun map-simple-funs (function)
+      (let ((function (%coerce-callable-to-fun function)))
+        (labels ((process (name value)
+                   (awhen (functoid-simple-fun value)
+                     (funcall function name it))))
+          (call-with-each-globaldb-name
+           (lambda (name)
+             ;; Methods are processed with their generic function
+             (unless (typep name '(cons (member sb-pcl::slow-method sb-pcl::fast-method)))
+               (let ((f (or (and (symbolp name) (macro-function name))
+                            (and (legal-fun-name-p name) (fboundp name)))))
+                 (typecase f
+                   (generic-function
+                    (loop for method in (sb-mop:generic-function-methods f)
+                          for fun = (sb-pcl::safe-method-fast-function method)
+                          when fun do (process (sb-kernel:%fun-name fun) fun)))
+                   (function
+                    (process name f)))))
+             #+sb-xref-for-internals
+             (let ((info (info :function :info name)))
+               (when info
+                 (loop for transform in (sb-c::fun-info-transforms info)
+                       for fun = (sb-c::transform-function transform)
+                       ;; Defined using :defun-only and a later %deftransform.
+                       unless (symbolp fun)
+                       do (process transform fun))))))
+          #+sb-xref-for-internals
+          (sb-int:dohash ((name vop) sb-c::*backend-template-names*)
+            (declare (ignore name))
+            (let ((fun (sb-c::vop-info-generator-function vop)))
+              (when fun
+                (process vop fun))))))))
+  (defun collect-xref (wanted-kind wanted-name)
+    (let ((result '()))
+      (map-simple-funs
+       (lambda (name fun)
+         (binding* ((xrefs (%simple-fun-xrefs fun) :exit-if-null))
+           (sb-c:map-packed-xref-data
+            (lambda (xref-kind xref-name xref-form-number)
+              (when (and (eq xref-kind wanted-kind)
+                         (equal xref-name wanted-name))
+                (let ((source-location (find-function-definition-source fun)))
+                  ;; Use the more accurate source path from the xref
+                  ;; entry.
+                  (setf (definition-source-form-number source-location)
+                        xref-form-number)
+                  (let ((name (cond ((sb-c::transform-p name)
+                                     (let ((fun-name (%fun-name fun)))
+                                       (append (if (consp fun-name)
+                                                   fun-name
+                                                   (list fun-name))
+                                               (let* ((type (sb-c::transform-type name))
+                                                      (type-spec (type-specifier type)))
+                                                 (and (sb-kernel:fun-type-p type)
+                                                      (list (second type-spec)))))))
+                                    ((sb-c::vop-info-p name)
+                                     (list 'sb-c:define-vop
+                                           (sb-c::vop-info-name name)))
+                                    (t
+                                     name))))
+                    (push (cons name source-location) result)))))
+            xrefs))))
+      result)))
 
 #+(and system-tlabs (not mark-region-gc))
 (progn
