@@ -24,6 +24,11 @@
         (remf args :hash-function)))
     (apply 'make-hash-table args)))
 
+(defun %hash-table-alist (hash-table &aux result)
+  (maphash (lambda (key value) (push (cons key value) result))
+           hash-table)
+  result)
+
 ;;; In correct code, TRULY-THE has only a performance impact and can
 ;;; be safely degraded to ordinary THE.
 (defmacro truly-the (type expr)
@@ -103,8 +108,6 @@
   (if (symbolp x) nil (error "Called SIMPLE-FUN-P on ~S" x)))
 (defun closurep (x)
   (if (symbolp x) nil (error "Called CLOSUREP on ~S" x)))
-(defun unbound-marker-p (x)
-  (if (symbolp x) nil (error "Called UNBOUND-MARKER-P on ~S" x)))
 (defun vector-with-fill-pointer-p (x)
   (if (symbolp x) nil (error "Called VECTOR-WITH-FILL-POINTER-P on ~S" x)))
 
@@ -124,16 +127,40 @@
        (or (not (typep x 'simple-array))
            (/= (array-rank x) 1))))
 
-;;; We maintain a separate GENSYM counter since the host is allowed to
-;;; mutate its counter however it wishes.
-(defvar sb-xc:*gensym-counter* 0)
-
+;;; While cross-compiling, we do not use *GENSYM-COUNTER* to make part
+;;; of the new symbol's name.  This is for two reasons: firstly,
+;;; genesis can coalesce uninterned symbols of similar names, which
+;;; leads to significant space savings in the resulting core;
+;;; secondly, it is very hard to maintain a consistent state of
+;;; *GENSYM-COUNTER*, even if we attempt to maintain our own: the
+;;; presence of (EVAL-WHEN (:COMPILE-TOPLEVEL ...) ...) in expansions
+;;; of e.g. SB-XC:DEFMACRO, SB-XC:DEFINE-COMPILER-MACRO and so on can
+;;; lead to macro-functions of target macros on the host, which then
+;;; will GENSYM varying number of times depending on whether they are
+;;; minimally-compiled or not: the only way to guarantee identical
+;;; symbol names at the same point in the compilation is not to use
+;;; *GENSYM-COUNTER* at all.
+;;;
+;;; Not using *GENSYM-COUNTER* would not be an issue under normal
+;;; circumstances, modulo the demands of the language spec: the only
+;;; thing that matters for the typical use of GENSYMs in code is the
+;;; identity of the symbol, not its name.  However, the coalescing of
+;;; uninterned symbols by genesis introduces a potential pitfall: if a
+;;; macro-writing-macro generates symbols at first use that the
+;;; newly-defined macro will use when *it* expands, the coalescing in
+;;; genesis would lead to that inner macro no longer having the
+;;; correct expansion.  (Or equivalents, such as macros expanded
+;;; inside definitions of functions declaimed INLINE).  In the event
+;;; of truly weird behaviour of some macro leading the intrepid
+;;; maintainer to this comment: try turning off uninterned symbol
+;;; coalescing in genesis by redefining GET-UNINTERNED-SYMBOL; if
+;;; things work after that, find the offending use of GENSYM and make
+;;; sure that everything has a distinct symbol name.  (This is not an
+;;; issue for user code, which is compiled with a normal
+;;; implementation of GENSYM).
 (defun sb-xc:gensym (&optional (thing "G"))
   (declare (type string thing))
-  (let ((n sb-xc:*gensym-counter*))
-    (prog1
-        (make-symbol (concatenate 'string thing (write-to-string n :base 10 :radix nil :pretty nil)))
-      (incf sb-xc:*gensym-counter*))))
+  (make-symbol thing))
 
 ;;; These functions are needed for constant-folding.
 (defun simple-array-nil-p (object)
@@ -150,12 +177,6 @@
 (defun %negate (number)
   (sb-xc:- number))
 
-(defun %single-float (number)
-  (coerce number 'single-float))
-
-(defun %double-float (number)
-  (coerce number 'double-float))
-
 (defun %ldb (size posn integer)
   (ldb (byte size posn) integer))
 
@@ -166,6 +187,22 @@
   (assert (typep array '(simple-array * (*))))
   (values array start end 0))
 
+;; We could probably just implement this by creating a displaced array
+;; if need be.
+(defmacro with-array-data (((data-var array &key offset-var)
+                            (start-var &optional (svalue 0))
+                            (end-var &optional (evalue nil))
+                            &key force-inline check-fill-pointer
+                                 array-header-p)
+                           &body forms
+                           &environment env)
+  (declare (ignore data-var array offset-var)
+           (ignore start-var svalue)
+           (ignore end-var evalue)
+           (ignore force-inline check-fill-pointer array-header-p)
+           (ignore forms env))
+  `(error "WITH-ARRAY-DATA not implemented on the host."))
+
 (defun %with-array-data/fp (array start end)
   (assert (typep array '(simple-array * (*))))
   (values array start end 0))
@@ -180,14 +217,11 @@
   (or (find-package string)
       (error "Cross-compiler bug: no package named ~S" string)))
 
-(defmacro without-package-locks (&body body)
-  `(progn ,@body))
-
 (defmacro with-single-package-locked-error ((&optional kind thing &rest format)
                                             &body body)
   (declare (ignore kind format))
-  `(let ((.dummy. ,thing))
-     (declare (ignore .dummy.))
+  `(progn
+     ,thing
      ,@body))
 
 (defun program-assert-symbol-home-package-unlocked (context symbol control)
@@ -247,6 +281,12 @@
 
 (in-package "SB-KERNEL")
 
+(define-symbol-macro *gc-epoch* 0)
+(deftype weak-vector () nil)
+(defun weak-vector-p (x) (declare (ignore x)) nil)
+(defun sb-thread:make-mutex (&key name) (list :mock-mutex name))
+(deftype sb-thread:mutex () '(cons (eql :mock-mutex)))
+
 ;;; These functions are required to emulate SBCL kernel functions
 ;;; in a vanilla ANSI Common Lisp cross-compilation host.
 ;;; The emulation doesn't need to be efficient, since it's needed
@@ -262,11 +302,6 @@
 (defun %instance-layout (instance)
   (declare (notinline classoid-layout))
   (classoid-layout (find-classoid (type-of instance))))
-(defun %instance-length (instance)
-  (declare (notinline layout-length))
-  ;; In the target, it is theoretically possible to have %INSTANCE-LENGTH
-  ;; exceeed layout length, but in the cross-compiler they're the same.
-  (layout-length (%instance-layout instance)))
 
 (defun %find-position (item seq from-end start end key test)
   (let ((position (position item seq :from-end from-end
@@ -332,6 +367,17 @@
 ;;;; in lieu of #+sb-xc-host elsewere which messes up toplevel form numbers.
 (in-package "SB-C")
 
+(defun allocate-weak-vector (n) (make-array (the integer n)))
+
+#+weak-vector-readbarrier
+(progn (deftype weak-vector () nil) ; nothing is a weak-vector
+       (defun sb-int:weak-vector-ref (v i)
+         (error "Called WEAK-VECTOR-REF on ~S ~S" v i))
+       (defun (setf sb-int:weak-vector-ref) (new v i)
+         (error "Called (SETF WEAK-VECTOR-REF) on ~S ~S ~S" new v i))
+       (defun sb-int:weak-vector-len (v)
+         (error "Called WEAK-VECTOR-LEN on ~S" v)))
+
 ;;; For macro lambdas that are processed by the host
 (declaim (declaration top-level-form))
 
@@ -403,3 +449,10 @@
 (defun range<<= (l x h) (and (< l x) (<= x h)))
 (defun range<=< (l x h) (and (<= l x) (< x h)))
 
+(defvar *unbound-marker* (make-symbol "UNBOUND-MARKER"))
+
+(defun make-unbound-marker ()
+  *unbound-marker*)
+
+(defun unbound-marker-p (x)
+  (eq x *unbound-marker*))
