@@ -1286,22 +1286,22 @@ relative to DEFAULTS."
            (type (member nil :host :device :directory :name :type :version)
                  field-key))
   (with-pathname (pathname pathname)
-    (flet ((frob (x)
-             (or (pattern-p x) (member x '(:wild :wild-inferiors)))))
-      (ecase field-key
-        ((nil)
-         (or (wild-pathname-p pathname :host)
-             (wild-pathname-p pathname :device)
-             (wild-pathname-p pathname :directory)
-             (wild-pathname-p pathname :name)
-             (wild-pathname-p pathname :type)
-             (wild-pathname-p pathname :version)))
-        (:host (frob (%pathname-host pathname)))
-        (:device (frob (%pathname-host pathname)))
-        (:directory (some #'frob (%pathname-directory pathname)))
-        (:name (frob (%pathname-name pathname)))
-        (:type (frob (%pathname-type pathname)))
-        (:version (frob (%pathname-version pathname)))))))
+    (labels ((wildp (x)
+               (or (pattern-p x) (if (member x '(:wild :wild-inferiors)) t nil)))
+             (test (field)
+               (wildp
+                (case field
+                  (:host (%pathname-host pathname)) ; always NIL
+                  (:device (%pathname-device pathname))
+                  (:directory
+                   (return-from test (some #'wildp (%pathname-directory pathname))))
+                  (:name (%pathname-name pathname))
+                  (:type (%pathname-type pathname))
+                  (:version (%pathname-version pathname))))))
+      (if (not field-key)
+          ;; SBCL does not allow :WILD in the host
+          (or (test :device) (test :directory) (test :name) (test :type) (test :version))
+          (test field-key)))))
 
 (defun pathname-match-p (in-pathname in-wildname)
   "Pathname matches the wildname template?"
