@@ -182,24 +182,30 @@
          (case from
            ((> >=) '<=))
          (case from
-           ((< <= . #.+eq-ops+) '<=))))
+           (< '<)
+           ((<= . #.+eq-ops+) '<=))))
     (>=
      (if not-p
          (case from
            ((< <=) '>=))
          (case from
-           ((> >= . #.+eq-ops+) '>=))))))
+           ((>) '>)
+           ((>= . #.+eq-ops+) '>=))))))
 
 (defun add-equality-constraint (operator first second constraints consequent-constraints alternative-constraints
                                 &optional (min-amount 0) max-amount)
   (case operator
-    (vector-length
+    ((vector-length vector-length<= vector-length>=)
      (let ((var (if (lambda-var-p first)
                     first
                     (lvar-dest-var first))))
        (when (and var
                   (lambda-var-constraints var))
-         (add-equality-constraint 'eq (make-vector-length-constraint var) second
+         (add-equality-constraint (case operator
+                                    (vector-length<= '<=)
+                                    (vector-length>= '>=)
+                                    (t 'eq))
+                                  (make-vector-length-constraint var) second
                                   constraints consequent-constraints alternative-constraints))))
     (t
      (let* ((x (cond ((lambda-var/vector-length-p first)
@@ -207,7 +213,9 @@
                      ((ok-lvar-lambda-var/vector-length first constraints))
                      (t
                       first)))
-            (y (cond ((lambda-var/vector-length-p second)
+            (y (cond ((ctype-p second)
+                      second)
+                     ((lambda-var/vector-length-p second)
                       second)
                      ((ok-lvar-lambda-var/vector-length second constraints))
                      (t
@@ -219,7 +227,9 @@
             (y-type (etypecase second
                       (lvar (lvar-type second))
                       (lambda-var (lambda-var-type second))
-                      (vector-length-constraint (specifier-type 'index)))))
+                      (vector-length-constraint (specifier-type 'index))
+                      (ctype
+                       second))))
        (when (lvar-p first)
          (constraint-propagate-back first operator second
                                     constraints
@@ -650,7 +660,8 @@
               (values))
       (give-up-ir1-transform)))
 
-(defoptimizer (vector-length equality-constraint) ((vector) node gen)
+
+(defun vector-constraint-length (vector gen)
   (let (type
         (vector-var (ok-lvar-lambda-var vector gen)))
     (when (and vector-var
@@ -685,8 +696,12 @@
                    (when (ctype-p y)
                      (setf type
                            (type-after-comparison op not-p (or type (specifier-type 'index)) y))))))))
-      (when type
-        (derive-node-type node type))))
+      type)))
+
+(defoptimizer (vector-length equality-constraint) ((vector) node gen)
+  (let ((type (vector-constraint-length vector gen)))
+    (when type
+      (derive-node-type node type)))
   :give-up)
 
 (deftransform %in-bounds-constraint ((vector length) * * :node node)
@@ -758,9 +773,9 @@
         do
         (add-equality-constraint operator var second constraints target nil min-amount max-amount))
   (when (lambda-var-p var)
-    (let ((vector-length (vector-length-var-p lvar constraints)))
+    (let ((vector-length (vector-length-var-p lvar constraints t)))
       (when vector-length
-        (conset-add-equality-constraint target 'eq var vector-length nil)))))
+        (add-equality-constraint 'eq var vector-length constraints target nil)))))
 
 (defun add-mv-let-result-constraints (call fun constraints &optional (target constraints))
   (let ((vars (lambda-vars fun))
@@ -775,7 +790,8 @@
   (let (gen)
     (loop for (operator second min-amount max-amount) in (lvar-result-constraints lvar constraints)
           do
-          (if (vector-length-constraint-p second)
+          (if (or (vector-length-constraint-p second)
+                  (ctype-p second))
               (conset-add-equality-constraint (or gen
                                                   (setf gen (make-conset)))
                                               operator var second nil min-amount)
@@ -873,11 +889,119 @@
   (list (list 'vector-length length)))
 
 (defoptimizer (%make-array constraint-propagate-result) ((length &rest args) node)
-  (list (list 'vector-length length)))
+  (when (csubtypep (lvar-type length) (specifier-type 'integer))
+    (list (list 'vector-length length))))
 
 (defoptimizer (make-sequence constraint-propagate-result) ((type length &rest args) node)
   (list (list 'vector-length length)))
 
+(defoptimizers constraint-propagate-result (remove delete remove-if delete-if remove-if-not delete-if-not)
+    ((x sequence &rest args &key from-end test test-not start end count key) node gen)
+  (let (c)
+    (let ((var (ok-lvar-lambda-var sequence gen)))
+      (when var
+        (push (list 'vector-length<= (make-vector-length-constraint var)) c)))
+    (let ((length (vector-length-type (lvar-type sequence))))
+      (when length
+       (push (list 'vector-length<= length)
+             c)))
+    c))
+
+(defun subseq-bounds (sequence start end gen)
+  (let* ((null-p (types-equal-or-intersect (lvar-type end) (specifier-type 'null)))
+         (end (if null-p
+                  (type-union (type-intersection (or (vector-length-type (lvar-type sequence))
+                                                     *universal-type*)
+                                                 (or (vector-constraint-length sequence gen)
+                                                     *universal-type*))
+                              (type-intersection (lvar-type end) (specifier-type 'integer)))
+                  (lvar-type end)))
+         (int-s (type-approximate-interval (lvar-type start)))
+         (int-e (type-approximate-interval end)))
+    (if (and int-s int-e)
+        (let* ((length (interval-sub int-e int-s))
+               (l (interval-low length))
+               (h (interval-high length)))
+          (values (if (typep l 'unsigned-byte)
+                      l
+                      0)
+                  h))
+        (values 0 nil))))
+
+(defoptimizer (vector-subseq* constraint-propagate-result) ((sequence start end) node gen)
+  (let (c
+        (null-p (types-equal-or-intersect (lvar-type end) (specifier-type 'null))))
+    (when (eql (lvar-type start) (specifier-type '(eql 0)))
+      (if null-p
+          (when (eql (lvar-type end) (specifier-type 'null))
+            (let ((seq-var (ok-lvar-lambda-var sequence gen)))
+              (when seq-var
+                (push (list 'vector-length (make-vector-length-constraint seq-var)) c))))
+          (push (list 'vector-length end) c)))
+    (multiple-value-bind (l h) (subseq-bounds sequence start end gen)
+      (when (or h
+                (> l 0))
+        (push (list 'vector-length (specifier-type `(integer ,l ,(or h '*))))
+              c)))
+    c))
+
+(defun concatenate-constraints (args gen &optional subseq)
+  (let ((min-sum 0)
+        (max t)
+        (max-sum 0)
+        r)
+    (loop while args
+          do (let ((arg (pop args)))
+               (cond ((and subseq
+                           (constant-lvar-p arg)
+                           (eq (lvar-value arg) 'sb-impl::%subseq))
+                      (multiple-value-bind (l h) (subseq-bounds (pop args) (pop args)
+                                                                (pop args) gen)
+                        (incf min-sum l)
+                        (if h
+                            (incf max-sum h)
+                            (setf max nil))))
+                     (t
+                      (let ((int (type-approximate-interval (type-intersection (or (vector-length-type (lvar-type arg))
+                                                                                   *universal-type*)
+                                                                               (or (vector-constraint-length arg gen)
+                                                                                   *universal-type*)))))
+                        (cond (int
+                               (incf min-sum (or (interval-low int) 0))
+                               (if (interval-high int)
+                                   (incf max-sum (interval-high int))
+                                   (setf max nil)))
+                              (t
+                               (setf max nil))))))))
+    (when (plusp min-sum)
+      (push (list 'vector-length>= (specifier-type `(eql ,min-sum)))
+            r))
+    (when (and max
+               (plusp max-sum))
+      (push (list 'vector-length<= (specifier-type `(eql ,max-sum)))
+            r))
+    r))
+
+(defoptimizers constraint-propagate-result
+    (%concatenate-to-simple-vector %concatenate-to-string %concatenate-to-base-string)
+    ((&rest seqs) node gen)
+  (concatenate-constraints seqs gen))
+
+(defoptimizers constraint-propagate-result
+    (%concatenate-to-vector)
+    ((widetag &rest seqs) node gen)
+  (concatenate-constraints seqs gen))
+
+(defoptimizers constraint-propagate-result
+    (%concatenate-to-string-subseq %concatenate-to-base-string-subseq %concatenate-to-simple-vector-subseq)
+    ((&rest seqs) node gen)
+  (concatenate-constraints seqs gen t))
+
+(defoptimizers constraint-propagate-result
+    (%concatenate-to-vector-subseq)
+    ((type &rest seqs) node gen)
+  (concatenate-constraints seqs gen t))
+
 (defoptimizer (read-sequence constraint-propagate-result) ((seq stream &key start end) node gen)
   (let ((var (ok-lvar-lambda-var seq gen)))
     (when var
@@ -887,3 +1011,265 @@
   (when (csubtypep (lvar-type y) (specifier-type '(real 0)))
     (values nil
             (list (list 1 '< y)))))
+
+(defoptimizers constraint-propagate-if (car cdr) ((list))
+  (values list (specifier-type 'cons) nil nil t))
+
+(defun find-position-item-type (sequence key test &optional test-not)
+  (let ((type *universal-type*))
+    (flet ((fun-accepts-type (fun-lvar argument)
+             (when fun-lvar
+               (let ((fun-type (lvar-fun-type fun-lvar t t)))
+                 (when (fun-type-p fun-type)
+                   (let ((arg (nth argument (fun-type-n-arg-types (1+ argument) fun-type))))
+                     (when arg
+                       (setf type
+                             (type-intersection type arg)))))))))
+      (fun-accepts-type test 0)
+      (fun-accepts-type test-not 0)
+      (when (and (or (not test)
+                     (lvar-fun-is test '(eq eql)))
+                 (not test-not))
+        (setf type
+              (type-intersection type (sequence-element-type sequence key)))))
+    type))
+
+(defun sequence-type-from-item (item-type)
+  (if (and item-type
+           (types-equal-or-intersect item-type (specifier-type '(or number character))))
+      (specifier-type '(not null))
+      (specifier-type '(and (not null) (or (not array) (vector t))))))
+
+(defoptimizer (%find-position constraint-propagate-if) ((item sequence from-end start end key test) node gen nth-value)
+  (let ((type (find-position-item-type sequence key test)))
+    (values nil nil
+            (list (list 'typep item (if (and (= nth-value 0)
+                                             (lvar-fun-is key '(identity))
+                                             (lvar-fun-is test '(eq eql equal equalp)))
+                                        (type-intersection type (specifier-type '(not null)))
+                                        type))
+                  (list 'typep sequence
+                        (sequence-type-from-item (single-value-type (node-derived-type node))))
+                  (let ((var (ok-lvar-lambda-var sequence gen)))
+                    (when var
+                      (list 'equality '> (make-vector-length-constraint var) (specifier-type '(eql 0)))))))))
+
+(defoptimizers constraint-propagate-if (position) ((item sequence &key key test test-not &allow-other-keys) node gen)
+  (values nil nil (list (list 'typep item (find-position-item-type sequence key test test-not))
+                        (list 'typep sequence
+                              (sequence-type-from-item (find-derive-type item sequence key test nil nil nil test-not)))
+                        (let ((var (ok-lvar-lambda-var sequence gen)))
+                          (when var
+                            (list 'equality '> (make-vector-length-constraint var) (specifier-type '(eql 0))))))))
+
+(defoptimizers constraint-propagate-if (find) ((item sequence &key key test test-not &allow-other-keys) node gen)
+  (let ((type (find-position-item-type sequence key test test-not)))
+    (values nil nil (list (list 'typep item
+                                (if (and (not key)
+                                         (not test-not)
+                                         (or (not test)
+                                             (lvar-fun-is test '(eq eql equal equalp))))
+                                    (type-intersection type (specifier-type '(not null)))
+                                    type))
+                          (list 'typep sequence
+                                (sequence-type-from-item (single-value-type (node-derived-type node))))
+                          (let ((var (ok-lvar-lambda-var sequence gen)))
+                            (when var
+                              (list 'equality '> (make-vector-length-constraint var) (specifier-type '(eql 0)))))))))
+
+(defoptimizers constraint-propagate-if (%member %member-eq %member-if %member-if-not) ((x list))
+  (values list (specifier-type 'cons) nil nil t))
+
+(defoptimizers constraint-propagate-if
+    (%member-key %member-key-eq) ((item list key))
+  (values nil nil (list (list 'typep list (specifier-type 'cons))
+                        (list 'typep item (find-position-item-type list key nil)))))
+
+(defoptimizers constraint-propagate-if
+    (%member-test %member-test-not) ((item list test))
+  (values nil nil (list (list 'typep list (specifier-type 'cons))
+                        (list 'typep item (find-position-item-type list nil test)))))
+
+(defoptimizers constraint-propagate-if
+    (%member-if-key %member-if-not-key) ((x list test))
+  (values list (specifier-type 'cons) nil nil t))
+
+(defoptimizers constraint-propagate-if (%member-key-test-not) ((item list key test))
+  (values nil nil (list (list 'typep list (specifier-type 'cons))
+                        (list 'typep item (find-position-item-type list key nil test)))))
+
+(defoptimizers constraint-propagate-if (%member-key-test) ((item list key test))
+  (values nil nil (list (list 'typep list (specifier-type 'cons))
+                        (list 'typep item (find-position-item-type list key test)))))
+
+(defun equal-length-constraints (x y gen)
+  (let ((x-var (ok-lvar-lambda-var x gen))
+        (y-var (ok-lvar-lambda-var y gen))
+        constraints)
+    (when (and x-var y-var)
+      (push (list 'equality 'eq (make-vector-length-constraint x-var)
+             (make-vector-length-constraint y-var))
+            constraints))
+    (flet ((lvar-type (var lvar)
+             (when var
+               (let ((type (lvar-type lvar)))
+                 (when (csubtypep type (specifier-type 'vector))
+                   (let ((type (vector-length-type type)))
+                     (when type
+                       (push (list 'equality 'eq (make-vector-length-constraint var) type) constraints))))))))
+      (lvar-type x-var y)
+      (lvar-type y-var x))
+    constraints))
+
+(defun equality-length-constraints (op x y gen)
+  (let ((x-var (ok-lvar-lambda-var x gen))
+        (y-var (ok-lvar-lambda-var y gen))
+        constraints)
+    (when (and x-var y-var)
+      (push (list 'equality op (make-vector-length-constraint x-var)
+                  (make-vector-length-constraint y-var))
+            constraints))
+    (flet ((lvar-type (op var lvar)
+             (when var
+               (let ((type (lvar-type lvar)))
+                 (when (csubtypep type (specifier-type 'vector))
+                   (let ((type (vector-length-type type)))
+                     (when type
+                       (push (list 'equality op (make-vector-length-constraint var) type) constraints))))))))
+      (lvar-type op x-var y)
+      (lvar-type (invert-operator op) y-var x))
+    constraints))
+
+(defoptimizers constraint-propagate-if (equal char=) ((x y) node gen)
+  (let ((x-type (equal-type (lvar-type x)))
+        (y-type (equal-type (lvar-type y))))
+    (let ((intersection (type-intersection x-type y-type)))
+      (unless (or (eq intersection *empty-type*)
+                  (eq intersection *universal-type*))
+        (let ((constraints))
+          (push (list 'typep x intersection) constraints)
+          (push (list 'typep y intersection) constraints)
+          (setf constraints (nconc constraints
+                                   (equal-length-constraints x y gen)))
+          (values nil nil constraints))))))
+
+(defoptimizers constraint-propagate-if (equalp bit-vector-= sb-impl::array-equalp) ((x y) node gen)
+  (let ((x-type (equalp-type (lvar-type x)))
+        (y-type (equalp-type (lvar-type y))))
+    (let ((intersection (type-intersection x-type y-type)))
+      (unless (or (eq intersection *empty-type*)
+                  (eq intersection *universal-type*))
+        (let ((constraints))
+          (push (list 'typep x intersection) constraints)
+          (push (list 'typep y intersection) constraints)
+          (setf constraints (nconc constraints
+                                   (equal-length-constraints x y gen)))
+          (values nil nil constraints))))))
+
+(defun character-set-range (lvar)
+  (if (constant-lvar-p lvar)
+      (let ((code (char-code (lvar-value lvar))))
+        (values code code))
+      (let ((type (lvar-type lvar)))
+        (if (typep type 'character-set-type)
+            (loop for (lo . hi) in (character-set-type-pairs type)
+                  minimize lo into min
+                  maximize hi into max
+                  finally (return (values min max)))
+            (values 0 (1- char-code-limit))))))
+
+(defun char<-constraints (char1 char2)
+  (multiple-value-bind (min2 max2) (character-set-range char2)
+    (values (list 'typep char1 (if (zerop max2)
+                                   *empty-type*
+                                   (specifier-type `(character-set ((0 . ,(1- max2)))))))
+            (list 'typep char1 (specifier-type `(character-set ((,min2 . #.(1- char-code-limit)))))))))
+
+(defun char>-constraints (char1 char2)
+  (multiple-value-bind (min2 max2) (character-set-range char2)
+    (values (list 'typep char1 (if (= min2 (1- char-code-limit))
+                                   *empty-type*
+                                   (specifier-type `(character-set ((,(1+ min2) . #.(1- char-code-limit)))))))
+            (list 'typep char1 (specifier-type `(character-set ((0 . ,max2))))))))
+
+(defoptimizer (char< constraint-propagate-if)
+    ((char1 char2))
+  (multiple-value-bind (if1 then1)
+      (char<-constraints char1 char2)
+    (multiple-value-bind (if2 then2)
+        (char>-constraints char2 char1)
+      (values nil nil (list if1 if2) (list then1 then2)))))
+
+(defoptimizer (char> constraint-propagate-if)
+    ((char1 char2))
+  (multiple-value-bind (if1 then1)
+      (char>-constraints char1 char2)
+    (multiple-value-bind (if2 then2)
+        (char<-constraints char2 char1)
+      (values nil nil (list if1 if2) (list then1 then2)))))
+
+(defun sequence-min-length (sequence start end)
+  (let* ((length (nth-value 1 (sequence-lvar-dimensions sequence)))
+         (start-int (and start
+                         (type-approximate-interval (lvar-type start))))
+         (end-int (and end
+                       (type-approximate-interval (lvar-type end))))
+         (start (if start
+                    (and start-int
+                         (interval-high start-int))
+                    0))
+         (end (if (or (not end)
+                      (eq (lvar-type end) (specifier-type 'null)))
+                  length
+                  (and end-int
+                       (interval-low end-int)))))
+    (when (and start end)
+      (- end start))))
+
+(defoptimizers constraint-propagate-if (string=* #+sb-unicode simple-character-string= simple-base-string=)
+    ((string1 string2 start1 end1 start2 end2) node gen)
+  (let ((min1 (sequence-min-length string1 start1 end1))
+        (min2 (sequence-min-length string2 start2 end2))
+        constraints)
+    (when (and min1
+               (> min1 1))
+      (push (list 'typep string2 (specifier-type '(not character))) constraints))
+    (when (and min2
+               (> min2 1))
+      (push (list 'typep string1 (specifier-type '(not character))) constraints))
+    (when (and (eql (lvar-type start1) (specifier-type '(eql 0)))
+               (eql (lvar-type start2) (specifier-type '(eql 0)))
+               (eql (lvar-type end1) (specifier-type 'null))
+               (eql (lvar-type end2) (specifier-type 'null)))
+     (setf constraints (nconc constraints
+                              (equal-length-constraints string1 string2 gen))))
+    (values nil nil constraints)))
+
+(defoptimizer (string-equal constraint-propagate-if) ((string1 string2 &key start1 end1 start2 end2) node gen)
+  (let ((min1 (sequence-min-length string1 start1 end1))
+        (min2 (sequence-min-length string2 start2 end2))
+        constraints)
+    (when (and min1
+               (> min1 1))
+      (push (list 'typep string2 (specifier-type '(not character))) constraints))
+    (when (and min2
+               (> min2 1))
+      (push (list 'typep string1 (specifier-type '(not character))) constraints))
+    (unless (or start1 start2 end1 end2)
+      (setf constraints (nconc constraints
+                               (equal-length-constraints string1 string2 gen))))
+    (values nil nil constraints)))
+
+(defoptimizer (search constraint-propagate-if) ((sub-sequence1 main-sequence2
+                                                               &key from-end test test-not key start1 start2 end1 end2)
+                                                node gen)
+  (let (constraints)
+    (unless (or start1 start2 end1 end2)
+      (setf constraints (equality-length-constraints '<= sub-sequence1 main-sequence2 gen)))
+    (values nil nil constraints)))
+
+(defoptimizer (%other-pointer-p constraint-propagate-if) ((x))
+  (values x (specifier-type 'other-pointer)))
+
+(defoptimizer (%array-rank= constraint-propagate-if) ((x n))
+  (values x (specifier-type `(array * ,(lvar-value n)))))
