@@ -1611,8 +1611,10 @@
    (lambda (arg)
      ;; Derive the bounds if the arg is in [-pi/2, pi/2].
      (trig-derive-type-aux arg
-                           (specifier-type `(float ,(sb-xc:- (sb-xc:/ pi 2))
-                                                   ,(sb-xc:/ pi 2)))
+                           (specifier-type `(or (double-float ,(sb-xc:- (sb-xc:/ pi 2))
+                                                              ,(sb-xc:/ pi 2))
+                                                ;; 1.5707964 coerced back to double-float is greater than (/ pi 2)
+                                                (single-float -1.5707963 1.5707963)))
                            #'tan
                            nil nil))
    #'tan))
@@ -2098,6 +2100,25 @@
   (def <=)
   (def >=))
 
+(macrolet ((def (op)
+             `(deftransform ,op ((x y) ((integer #.most-negative-exactly-single-float-integer
+                                                 #.most-positive-exactly-single-float-integer)
+                                        float)
+                                 * :node node :important nil
+                                   :policy (> speed 1))
+                (unless (and (types-equal-or-intersect (lvar-type y) (specifier-type 'double-float))
+                             (types-equal-or-intersect (lvar-type y) (specifier-type 'single-float)))
+                  (give-up-ir1-transform))
+                (delay-ir1-transform node :ir1-phases)
+                `(if (single-float-p y)
+                     (,',op (%single-float x) (truly-the single-float y))
+                     (,',op (%double-float x) (truly-the double-float y))))))
+  (def =)
+  (def <)
+  (def >)
+  (def <=)
+  (def >=))
+
 (deftransform phase ((n))
   (splice-fun-args n 'complex 2)
   `(lambda (x y)
