@@ -244,8 +244,10 @@
   ((or index list) object-not-index-or-list)
   condition
   sb-pcl::fast-method-call
-  ((or symbol string) object-not-or-symbol-string)
-  ((and unsigned-byte fixnum) object-not-unsigned-fixnum)))
+  ((or symbol string) object-not-symbol-or-string)
+  ((or symbol string character) object-not-string-designator)
+  ((and unsigned-byte fixnum) object-not-unsigned-fixnum)
+  bit-index))
 
 (defun error-number-or-lose (name)
   (or (position name sb-c:+backend-internal-errors+
