@@ -60,9 +60,6 @@
           ;; the DEFCONSTANT of the same name, hence the suffix.
           &key ((:dynamic-space-start dynamic-space-start*))
                ((:dynamic-space-size dynamic-space-size*))
-               ;; The immobile-space START parameters should not be used
-               ;; except in forcing discontiguous addresses for testing.
-               ;; And of course, don't use them if unsupported.
                ((:fixedobj-space-start fixedobj-space-start*))
                ((:fixedobj-space-size  fixedobj-space-size*) (* 48 1024 1024))
                ((:text-space-start text-space-start*))
@@ -97,7 +94,6 @@
                            (static-code ,small-space-size))
                          #+immobile-space
                          `((fixedobj ,fixedobj-space-size*)
-                           (alien-linkage ,alien-linkage-space-size)
                            (text ,text-space-size*))))
          (ptr small-spaces-start)
          (small-space-forms
@@ -107,7 +103,6 @@
                           (member space '(fixedobj text permgen
                                           #+relocatable-static-space safepoint
                                           #+relocatable-static-space static
-                                          #+immobile-space alien-linkage
                                           read-only)))
                         (start ptr)
                         (end (+ ptr size)))
@@ -228,6 +223,8 @@
     *immobile-codeblob-tree* ; for generations 0 through 5 inclusive
     *immobile-codeblob-vector* ; for pseudo-static-generation
     *dynspace-codeblob-tree*
+    *linkage-name-map*
+    *elf-linkage-cell-modified*
     sb-impl::**finalizer-store**
     sb-impl::*finalizer-rehashlist*
     sb-impl::*finalizers-triggered*
@@ -303,7 +300,7 @@
 |#
 
 (defconstant-eqx common-static-fdefns
-    '(;; This is the standard set of assembly routines that need to call into lisp.
+    `(;; This is the standard set of assembly routines that need to call into lisp.
       ;; A few backends add TWO-ARG-/= and others to this, in their {arch}/parms
       two-arg-+
       two-arg--
@@ -312,6 +309,7 @@
       two-arg-<
       two-arg->
       two-arg-=
+      #-linkage-space ,@'(
       eql
       %negate
       ;; These next ones are not called from assembly code, but from lisp.
@@ -337,7 +335,7 @@
       vector-hairy-data-vector-set/check-bounds
       vector-hairy-data-vector-ref/check-bounds
       %ldb
-      vector-unsigned-byte-8-p)
+      vector-unsigned-byte-8-p))
   #'equalp)
 
 ;;; Refer to the lengthy comment in 'src/runtime/interrupt.h' about
@@ -427,6 +425,8 @@
        (defconstant n-symbol-hash-discard-bits
          (let ((precision (+ 32 n-symbol-hash-prng-bits))) ; total N bits
            (- 64 precision)))
+       ;; Allow .5 million global names, expandable to 4 million (22 bits)
+       (defconstant n-linkage-index-bits (or #+linkage-space 19 0))
        (defconstant-eqx sb-impl::symbol-hash-prng-byte
          (byte n-symbol-hash-prng-bits (- 32 n-symbol-hash-prng-bits))
          #'equal))
