@@ -202,28 +202,6 @@
 (def-two-arg-funs (number number)
   /=)
 
-;;; A list of function which always call their functional argument correctly,
-;;; meaning that no arg-count-error can occur in the callee
-;;; and therefore the callee can skip the check.
-(dolist (fun '(sb-thread::call-with-mutex
-               sb-thread::call-with-recursive-lock
-               sb-thread::call-with-system-mutex
-               sb-thread::call-with-system-mutex/allow-with-interrupts
-               sb-thread::call-with-system-mutex/without-gcing
-               sb-thread::call-with-recursive-system-lock
-               ;; wish we had a little more commonality to these names
-               sb-impl::%with-standard-io-syntax
-               sb-impl::%with-rebound-io-syntax
-               sb-debug::funcall-with-debug-io-syntax
-               sb-impl::call-with-sane-io-syntax
-               ;; there are others, maybe %with-compilation-unit
-               sb-impl::%print-unreadable-object
-               ))
-  (let ((info (info :function :info fun)))
-    (when info
-      (setf (ir1-attributep (fun-info-attributes info) callee-omit-arg-count-check)
-            t))))
-
 ;;; Unwrap predicates to enable tail calls
 (defun unwrap-predicates (combination)
   (let ((info (combination-fun-info combination)))
@@ -293,7 +271,9 @@
                                           (translate-two-args (global-var-%source-name leaf)))))
                                  (*compiler-error-context* node))
                             (and replacement
-                                 (find-global-fun replacement t)))))
+                                 (prog1
+                                     (find-global-fun replacement t)
+                                   (record-late-xref :calls replacement ref))))))
                  (cond ((ref-p ref)
                         (let ((replacement (translate ref)))
                           (when replacement
@@ -308,26 +288,6 @@
                                 (setf (node-derived-type cast)
                                       (lvar-derived-type (cast-value cast)))))))))))))
          node)
-        ;; One more thing: builtin higher-order functions utilized by builtin macros
-        ;; can impart a policy change to the callee, but it can't (easily) be done
-        ;; strictly lexically, because the policy would leak downward.
-        ;; e.g. (WITH-SOME-STUFF () ..) ->
-        ;; -> (DX-FLET ((THUNK () <user-code>)) (CALL-WITH-STUFF #'THUNK))
-        ;; should not inject (OPTIMIZE (VERIFY-ARG-COUNT 0)) at the top of <user-code>
-        ;; because every lambda therein would omit the arg-count check.
-        (when (ir1-attributep (fun-info-attributes (combination-fun-info node))
-                              callee-omit-arg-count-check)
-          (let* ((args (combination-args node))
-                 (arg (if (eq (lvar-fun-name (combination-fun node) t)
-                              'sb-impl::%print-unreadable-object)
-                          (fourth args)
-                          (first args)))
-                 (ref (and arg (lvar-uses arg))))
-            (when (and (ref-p ref) (lambda-p (ref-leaf ref)))
-              ;; It seems impolite (un-debuggable too) to alter the lexenv-policy
-              ;; of functional-lexenv, so annotate this differently.
-              (setf (getf (functional-plist (ref-leaf ref)) 'verify-arg-count)
-                    nil))))
         (when-vop-existsp (:named sb-vm::load-other-pointer-widetag)
           (reorder-type-tests node))))))
 
@@ -344,19 +304,37 @@
 (defun rewrite-full-call (combination)
   (let ((combination-name (lvar-fun-name (combination-fun combination) t))
         (args (combination-args combination)))
-    (let ((two-arg (assoc combination-name *two-arg-functions*)))
-      (when (and two-arg
-                 (= (length args) 2)
-                 (not (fun-lexically-notinline-p combination-name
-                                                 (node-lexenv combination))))
-        (destructuring-bind (name two-arg &optional types typed-two-arg) two-arg
-          (declare (ignore name))
-          (when (and types
-                     (loop for arg in args
-                           for type in types
-                           always (csubtypep (lvar-type arg) type)))
-            (setf two-arg typed-two-arg))
-          (change-full-call combination two-arg))))))
+    (when (eq (combination-kind combination) :known)
+      (let ((two-arg (assoc combination-name *two-arg-functions*)))
+        (when (and two-arg
+                   (= (length args) 2))
+          (destructuring-bind (name two-arg &optional types typed-two-arg) two-arg
+            (declare (ignore name))
+            (when (and types
+                       (loop for arg in args
+                             for type in types
+                             always (csubtypep (lvar-type arg) type)))
+              (setf two-arg typed-two-arg))
+            (change-full-call combination two-arg))))
+      (let ((lvar (node-lvar combination)))
+        (when (or (lvar-single-value-p lvar)
+                  (mv-bind-unused-p lvar 1))
+          (let ((single-value-fun (getf '(truncate sb-kernel::truncate1
+                                          floor sb-kernel::floor1
+                                          ceiling sb-kernel::ceiling1)
+                                        combination-name)))
+            (when single-value-fun
+              (unless (cdr args)
+                (let* ((leaf (find-constant 1))
+                       (ref (make-ref leaf))
+                       (lvar (make-lvar combination)))
+                  (use-lvar ref lvar)
+                  (push ref (leaf-refs leaf))
+                  (insert-ref-before ref combination-name)
+                  (setf (cdr args) (list lvar))))
+              (change-full-call combination single-value-fun)
+              (setf (node-derived-type combination)
+                    (make-single-value-type (single-value-type (node-derived-type combination)))))))))))
 
 ;;; The %other-pointer-subtype-p optimizer in ir2opt combines multiple
 ;;; checks for other-pointer into a single widetag load.
@@ -459,10 +437,10 @@
 (defun ir1-finalize (component)
   (declare (type component component))
   (dolist (fun (component-lambdas component))
-    (case (functional-kind fun)
-      (:external
+    (functional-kind-case fun
+      (external
        (finalize-xep-definition fun))
-      ((nil :toplevel)
+      ((nil toplevel)
        (setf (leaf-type fun) (definition-type fun)))))
 
   (maphash #'note-failed-optimization
